package com.ustadmobile.core.db.dao

import com.ustadmobile.door.DoorDatabase
import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.ext.prepareAndUseStatement
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.lib.db.entities.Role
import kotlin.Boolean
import kotlin.IllegalArgumentException
import kotlin.Long
import kotlin.Unit
import kotlin.collections.List

public class RoleDao_JdbcKt(
  public val _db: DoorDatabase
) : RoleDao() {
  public val _insertAdapterRole_: EntityInsertionAdapter<Role> = object :
      EntityInsertionAdapter<Role>(_db) {
    public override fun makeSql(returnsId: Boolean) = when(dbType) {
      DoorDbType.SQLITE -> {
        "INSERT INTO Role (roleUid, roleName, roleActive, roleMasterCsn, roleLocalCsn, roleLastChangedBy, roleLct, rolePermissions) VALUES(?, ?, ?, ?, ?, ?, ?, ?)"
      }
      DoorDbType.POSTGRES ->  {
        "INSERT INTO Role (roleUid, roleName, roleActive, roleMasterCsn, roleLocalCsn, roleLastChangedBy, roleLct, rolePermissions) VALUES(COALESCE(?,nextval('Role_roleUid_seq')), ?, ?, ?, ?, ?, ?, ?)" + if(returnsId) { " RETURNING roleUid" } else "" 
      }
      else -> {
        throw IllegalArgumentException("Unsupported db type")
      }
    }

    public override fun bindPreparedStmtToEntity(stmt: PreparedStatement, entity: Role): Unit {
      if(entity.roleUid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.roleUid)
      }
      stmt.setString(2, entity.roleName)
      stmt.setBoolean(3, entity.roleActive)
      stmt.setLong(4, entity.roleMasterCsn)
      stmt.setLong(5, entity.roleLocalCsn)
      stmt.setInt(6, entity.roleLastChangedBy)
      stmt.setLong(7, entity.roleLct)
      stmt.setLong(8, entity.rolePermissions)
    }
  }

  public override fun insert(entity: Role): Long {
    val _retVal = _insertAdapterRole_.insertAndReturnId(entity)
    return _retVal
  }

  public override suspend fun insertAsync(entity: Role): Long {
    val _retVal = _insertAdapterRole_.insertAndReturnIdAsync(entity)
    return _retVal
  }

  public override fun insertList(entityList: List<out Role>): Unit {
    _insertAdapterRole_.insertList(entityList)
  }

  public override fun updateList(entityList: List<out Role>): Unit {
    val _sql =
        "UPDATE Role SET roleName = ?, roleActive = ?, roleMasterCsn = ?, roleLocalCsn = ?, roleLastChangedBy = ?, roleLct = ?, rolePermissions = ? WHERE roleUid = ?"
    _db.prepareAndUseStatement(_sql) {
       _stmt ->
      _stmt.getConnection().setAutoCommit(false)
      for(_entity in entityList) {
        _stmt.setString(1, _entity.roleName)
        _stmt.setBoolean(2, _entity.roleActive)
        _stmt.setLong(3, _entity.roleMasterCsn)
        _stmt.setLong(4, _entity.roleLocalCsn)
        _stmt.setInt(5, _entity.roleLastChangedBy)
        _stmt.setLong(6, _entity.roleLct)
        _stmt.setLong(7, _entity.rolePermissions)
        _stmt.setLong(8, _entity.roleUid)
        _stmt.executeUpdate()
      }
      _stmt.getConnection().commit()
    }
  }

  public override fun update(entity: Role): Unit {
    val _sql =
        "UPDATE Role SET roleName = ?, roleActive = ?, roleMasterCsn = ?, roleLocalCsn = ?, roleLastChangedBy = ?, roleLct = ?, rolePermissions = ? WHERE roleUid = ?"
    _db.prepareAndUseStatement(_sql) {
       _stmt ->
      _stmt.setString(1, entity.roleName)
      _stmt.setBoolean(2, entity.roleActive)
      _stmt.setLong(3, entity.roleMasterCsn)
      _stmt.setLong(4, entity.roleLocalCsn)
      _stmt.setInt(5, entity.roleLastChangedBy)
      _stmt.setLong(6, entity.roleLct)
      _stmt.setLong(7, entity.rolePermissions)
      _stmt.setLong(8, entity.roleUid)
      _stmt.executeUpdate()
    }
  }
}
